include<NecksCaps.scad>

/* [Basic] */
Spike_Start_Diameter = 12;
Spike_Finish_Diameter = 1;
Spike_Length = 75;
//How thick the "Walls" Of the Spike Should Be. A Multiple Of Your Line Width (Default 0.45mm For Prusa Slicer) Works Well, 1.35 Gives you A thickness Of 3 Perimeters
Wall_Thickness = 1.35;

//How Many Rows Of Water Holes
Hole_Rows = 3;
//How Many Water Holes In Each Row
Holes_Per_Row = 4;
//The Size Of The Holes
Hole_Size = 1.25;

//Include Bumps On the Connector For Better Grip
Connector_Grip = true;
//Include A Text Label Around The Connector, Must Disable Connector Grip
Connector_Label = false;
//Label Text, Capitals Are Stongly Suggested
Label_Text = "LETTUCE";
//Font Size Of Text
Label_Size = 6;
Label_Spacing = 1; //  [0:0.1:2]

Shoulder = false;
Shoulder_Height = 4;
Shoulder_Diameter = 32;

/* [Advanced] */
//Detail of rounded parts
$fn = 60;

//Majorly Speed Up Rendering By Using A Pre Rendered Thread Mesh
Use_Pre_Rendered_Thread  = true;

connectorHeight = 20;

connectorDiameter = 18;

threadHeight = 1400;

connectorGripDepth = 0.5;

connectorGripWidth = 1;

numberOfGripGrooves = 20;

joinerHeight = 15;

module shoulder()
{
    if (Shoulder == true)
    {
        translate([0, 0, 12])
        {
            difference()
            {
                cylinder(Shoulder_Height, connectorDiameter-4, connectorDiameter-4);
                translate([0, 0, -1]) cylinder(Shoulder_Height+2, connectorDiameter-8, connectorDiameter-8);
            }
        }
    }
}

module thread()
{
    translate([0, 0, -2.8]) 
    {
        if (Use_Pre_Rendered_Thread==true)
        {
            import("28PCO1881_Pre_Rendered.stl");
        }
        else
        {
            28PCO1881(threadHeight); // Library by Xavan https://www.thingiverse.com/thing:1654620
        }
    }
}

module connectorGrip()
{
    difference()
    {
        intersection()
        {
            cylinder(connectorHeight, connectorDiameter+connectorGripDepth, connectorDiameter+connectorGripDepth);
            for (i=[1:numberOfGripGrooves/2])
            {
                rotate([0, 0, (180/(numberOfGripGrooves/2))*i]) translate([-connectorGripWidth/2, -50, -1]) cube([connectorGripWidth, 100, 100]);
            }
        }
        
        translate([0, 0, -1]) cylinder(connectorHeight+2, connectorDiameter, connectorDiameter);   
    }
}

module connector()
{
    difference()
    {
        cylinder(connectorHeight, connectorDiameter, connectorDiameter);
        translate([0, 0, -1]) cylinder(connectorHeight+2, 25.07/2+1.7, 25.07/2+1.7);
    }
    
    if (Connector_Grip==true) connectorGrip();
        
    color("Red") translate([0, 0, threadHeight/113]) 
    {
        difference()
        {
            cylinder(connectorHeight-threadHeight/113, connectorDiameter-1, connectorDiameter-1);
            translate([0, 0, -1]) cylinder(connectorHeight-threadHeight/113+2, 25.07/2-2, 25.07/2-2);
            
            cylinder(2, 25.07/2+1.7, 25.07/2-2);
        }
    }
}

module joiner()
{
    difference()
    {
        cylinder(joinerHeight, connectorDiameter, Spike_Start_Diameter);
        cylinder(joinerHeight, connectorDiameter-Wall_Thickness, Spike_Start_Diameter-Wall_Thickness);
    }
}

module spike()
{
    difference()
    {
        cylinder(Spike_Length, Spike_Start_Diameter, Spike_Finish_Diameter);
        cylinder(Spike_Length-Wall_Thickness, Spike_Start_Diameter-Wall_Thickness, 0);
        spikeHoles();
    }
}

module spikeRoundedEnd()
{
    sphere(Spike_Finish_Diameter);
}

module spikeHoles()
{
    for (row=[1:Hole_Rows])
    {
        for (column=[1:Holes_Per_Row])
        {
            translate([0, 0, (Spike_Length/(Holes_Per_Row+2))*column]) rotate([0, 90, (360/Hole_Rows)*row]) cylinder(100, Hole_Size, Hole_Size);
        }
    }
}

module label()
{
    for(i=[0:len(Label_Text)]){
      rotate([0,0,((i-(len(Label_Text)/2))*Label_Size*(3.2*Label_Spacing))]){
        translate( [connectorDiameter-1,0,0])
          rotate([90,180,90])
            linear_extrude(1)
              text(Label_Text[len(Label_Text)-i],valign="center", halign="center", Label_Size, font="Arial:style=Bold");
        }
     }
}

module main()
{
    thread();
    
    difference()
    {
        connector();
        if (Connector_Label==true && Connector_Grip==false) translate([0, 0, connectorHeight/2]) label();
    }
    
    translate([0, 0, connectorHeight]) joiner();
    translate([0, 0, connectorHeight+joinerHeight]) spike();
    translate([0, 0, connectorHeight+joinerHeight+Spike_Length-0.1
    ]) spikeRoundedEnd();
    
    color("#FF0000") shoulder();
}

main();